﻿using Syncfusion.DocIO.DLS;
using Syncfusion.DocIORenderer;
using Syncfusion.Pdf;
using System;
using System.Collections.Generic;
using System.IO;
using SkiaSharp;


namespace WordToPDF_Example
{
    class Program
    {
        static  CustomFonts customFonts = new CustomFonts();
        
        static void Main(string[] args)
        {
            //Set the custom folder path to load font stream.
            customFonts.Init(@"Fonts/");

            FileStream stream = new FileStream(@"Template.docx", FileMode.Open, FileAccess.ReadWrite);
            //Creates a new Word document.
            WordDocument wordDocument = new WordDocument(stream, Syncfusion.DocIO.FormatType.Docx);

            //Initializes the SubstituteFont event to perform font substitution during Word to PDF conversion.
            wordDocument.FontSettings.SubstituteFont += new SubstituteFontEventHandler(SubstituteFont);

            //Create instance for DocIORenderer for Word to PDF conversion
            DocIORenderer render = new DocIORenderer();
            //Converts Word document to PDF.
            PdfDocument pdfDocument = render.ConvertToPDF(wordDocument);
            //Release the resources used by the Word document and DocIO Renderer objects.
            render.Dispose();
            wordDocument.Dispose();
            //Saves the PDF file.
            FileStream outputStream = new FileStream(@"Output.pdf", FileMode.CreateNew, FileAccess.Write);
            pdfDocument.Save(outputStream);
            //Closes the instance of PDF document object.
            pdfDocument.Close();
            //Dispose the instance of FileStream.
            outputStream.Dispose();
            //Dispose the collection.
            customFonts.Dispose();

        }


        /// <summary>
        /// Substitutes the font.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="args">Arguments.</param>

        private static void SubstituteFont(object sender, SubstituteFontEventArgs args)
        {
           
            //Sets the font if the specified font is not installed in the production environment.
            if (customFonts.ContainsKey(args.OriginalFontName))
            {
                //Set the alternate font stream from the font in custom folder.
               args.AlternateFontStream = customFonts[args.OriginalFontName];
            }
           
        }
       
    }
    /// <summary>
    /// Represents a class to maintain fonts from custom folder.
    /// </summary>
    class CustomFonts
    {
        private static Dictionary<string, FileStream> _fonts;

        public Stream this[string font]
        {
            get
            {
                if (_fonts == null)
                    return null;
                else
                {
                 Stream stream = new MemoryStream();
                 _fonts[font].Position = 0;
                 _fonts[font].CopyTo(stream);
                return stream;
                }
            }
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="WordToPDF_Example.CustomFonts"/> class.
        /// </summary>
        public CustomFonts()
        {
            _fonts = new Dictionary<string, FileStream>();
        }

        /// <summary>
        /// Loads the font from given folder path.
        /// </summary>
        /// <param name="path">Path.</param>
        internal void Init(string path)
        {
            DirectoryInfo directoryInfo = new DirectoryInfo(path);
            SKTypeface typeface = null;
            //Get all fonts in folder.
            foreach (FileInfo file in directoryInfo.GetFiles())
            {
                string extension = file.Extension.ToLower();
                if (extension.Equals(".ttf") || extension.Equals(".otf"))
                {
                    //Loads the font and maintain in collection using it typeface.
                    System.IO.FileStream fontFile = file.Open(FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
                    typeface = SKTypeface.FromStream(file.Open(FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite));
                    if(!_fonts.ContainsKey(typeface.FamilyName))
                        _fonts.Add(typeface.FamilyName, fontFile);
                }
            }
        }
        /// <summary>
        /// Check whether the collection contains given font name.
        /// </summary>
        /// <returns><c>true</c>, if key was containsed, <c>false</c> otherwise.</returns>
        /// <param name="fontName">Font name.</param>
        internal bool ContainsKey(string fontName)
        {
            if (_fonts != null && _fonts.ContainsKey(fontName))
                return true;
            return false;
        }
        /// <summary>
        /// Remove the specified fontName.
        /// </summary>
        /// <param name="fontName">Name of the Font</param>
        internal void Remove(string fontName)
        {
            if (_fonts != null)
                _fonts.Remove(fontName);
        }
        /// <summary>
        /// Clear the instance.
        /// </summary>
        internal void Clear()
        {
            if(_fonts != null)
                _fonts.Clear();
        }
        /// <summary>
        /// Releases all resource used by the <see cref="WordToPDF_Example.CustomFonts"/> object.
        /// </summary>
        internal void Dispose()
        {
            Clear();
            _fonts = null;
        }
    }
}
